// JsRTDemo.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "jsrt.h"
#include "assert.h"
#include <string>

void PrepareJavascriptRuntime(JsRuntimeHandle& runtime)
{
	JsErrorCode err;

	err = JsCreateRuntime(JsRuntimeAttributeNone, JsRuntimeVersion11, nullptr, &runtime);
	assert(JsNoError == err);

	JsContextRef context;
	err = JsCreateContext(runtime, nullptr, &context);
	assert(JsNoError == err);

	err = JsSetCurrentContext(context);
	assert(JsNoError == err);
}

void DisposeJavascriptRuntime(const JsRuntimeHandle& runtime)
{
	JsErrorCode err;

	err = JsSetCurrentContext(JS_INVALID_REFERENCE);
	assert(JsNoError == err);

	err = JsDisposeRuntime(runtime);
	assert(JsNoError == err);
}

void RunScript()
{
	JsErrorCode err;

	std::wstring script(L"native.printf('number=%#x string=%s\\n', 255, 'test')");
	JsValueRef result;
	JsSourceContext contextCookie = 0;

	err = JsRunScript(script.c_str(), contextCookie, L"source", &result);
	assert(JsNoError == err);
}

JsValueRef CALLBACK PrintFormat(JsValueRef callee, bool isConstructCall, JsValueRef *arguments, unsigned short argumentCount, void *callbackState)
{
	JsErrorCode err;

	const wchar_t *format;
	size_t length;
	err = JsStringToPointer(arguments[1], &format, &length);
	assert(JsNoError == err);

	VARIANT variant;
	err = JsValueToVariant(arguments[2], &variant);
	assert(JsNoError == err);

	const wchar_t *str;
	err = JsStringToPointer(arguments[3], &str, &length);
	assert(JsNoError == err);

	wprintf(format, variant.intVal, str);

	return JS_INVALID_REFERENCE;
}

void PrepareNativeObject()
{
	JsErrorCode err;

	JsValueRef global;
	err = JsGetGlobalObject(&global);
	assert(JsNoError == err);

	JsPropertyIdRef nativeProp;
	err = JsGetPropertyIdFromName(L"native", &nativeProp);
	assert(JsNoError == err);

	JsValueRef nativeObj;
	err = JsCreateObject(&nativeObj);
	assert(JsNoError == err);

	JsPropertyIdRef printfProp;
	err = JsGetPropertyIdFromName(L"printf", &printfProp);
	assert(JsNoError == err);

	JsValueRef printfFunc;
	err = JsCreateFunction(PrintFormat, nullptr, &printfFunc);
	assert(JsNoError == err);

	err = JsSetProperty(nativeObj, printfProp, printfFunc, true);
	assert(JsNoError == err);

	err = JsSetProperty(global, nativeProp, nativeObj, true);
	assert(JsNoError == err);
}

int _tmain(int argc, _TCHAR* argv[])
{
	JsRuntimeHandle runtime;

	PrepareJavascriptRuntime(runtime);

	PrepareNativeObject();

	RunScript();

	DisposeJavascriptRuntime(runtime);

	return 0;
}

